searchData={"content_type":"text/markdown","items":[{"doc":"Provides a high-level interface for text chunking, employing a configurable splitting strategy (defaults to recursive splitting).  Manages options and coordinates the process, tracking chunk metadata.\n\n**Key Features**\n\n* **Customizable Splitting:**  Allows the splitting strategy to be customized via the `:strategy` option.\n* **Size and Overlap Control:**  Provides options for `:chunk_size` and `:chunk_overlap`.\n* **Metadata Tracking:**  Generates `Chunk` structs containing byte range information.\n\n**Supported Options**\n* `:chunk_size` (positive integer, default: 2000) - Maximum size in code point length for each chunk.\n* `:chunk_overlap` (non-negative integer, default: 200) - Number of overlapping code points between consecutive chunks to preserve context.\n* `:strategy` (module default: `RecursiveChunk`) - A module implementing the split function. Currently only `RecursiveChunk` is supported.\n* `:format` (atom, default: `:plaintext`) - The format of the input text. Used to determine where to split the text in some strategies.","ref":"TextChunker.html","title":"TextChunker","type":"module"},{"doc":"Splits the provided text into a list of `%Chunk{}` structs.","ref":"TextChunker.html#split/2","title":"TextChunker.split/2","type":"function"},{"doc":"```elixir\niex> long_text = \"This is a very long text that needs to be split into smaller pieces for easier handling.\"\niex> TextChunker.split(long_text)\n# => [%Chunk{}, %Chunk{}, ...]\n```\n\niex> TextChunker.split(long_text, chunk_size: 10, chunk_overlap: 3)\n# => Generates many smaller chunks with significant overlap","ref":"TextChunker.html#split/2-examples","title":"Examples - TextChunker.split/2","type":"function"},{"doc":"Defines the `Chunk` struct, representing a contiguous block of text extracted during the splitting process. It stores the text content along with its corresponding byte range within the original input text.","ref":"TextChunker.Chunk.html","title":"TextChunker.Chunk","type":"module"},{"doc":"","ref":"TextChunker.Chunk.html#t:t/0","title":"TextChunker.Chunk.t/0","type":"type"},{"doc":"Defines the contract that must be implemented for all text splitting strategies.","ref":"TextChunker.ChunkerBehaviour.html","title":"TextChunker.ChunkerBehaviour","type":"behaviour"},{"doc":"","ref":"TextChunker.ChunkerBehaviour.html#c:split/2","title":"TextChunker.ChunkerBehaviour.split/2","type":"callback"},{"doc":"Handles recursive text splitting, aiming to adhere to configured size and overlap requirements.\nEmploys a flexible separator-based approach to break down text into manageable chunks, while generating metadata for each produced chunk.\n\n**Key Features:**\n\n* **Size-Guided Chunking:** Prioritizes splitting text into semantic blocks while respecting the maximum `chunk_size`.\n* **Context Preservation:** Maintains `chunk_overlap` to minimize information loss at chunk boundaries.\n* **Separator Handling:** Selects the most appropriate delimiter (e.g., line breaks, spaces) based on the text content.\n* **Metadata Generation:** Creates `%TextChunker.Chunk{}` structs containing the split text and its original byte range.\n* **Oversized Chunk Warnings:**  Provides feedback when chunks cannot be created due to misconfiguration or limitations of the input text.\n\n**Algorithm Overview**\n\n1. **Separator Prioritization:**  Establishes a list of potential separators (e.g., line breaks, spaces), ordered by their expected relevance to the text structure.\n2. **Recursive Splitting:**\n  *  Iterates through the separator list.\n  *  Attempts to split the text using the current separator.\n  *  If a split is successful, recursively applies the algorithm to any resulting sub-chunks that still exceed the `chunk_size`.\n3. **Chunk Assembly:**\n  *  Combines smaller text segments into chunks, aiming to get as close to the `chunk_size` as possible.\n  *  Employs `chunk_overlap` to ensure smooth transitions between chunks.\n4. **Metadata Generation:**  Tracks byte ranges for each chunk for potential reassembly of the original text.","ref":"TextChunker.Strategies.RecursiveChunk.html","title":"TextChunker.Strategies.RecursiveChunk","type":"module"},{"doc":"","ref":"TextChunker.Strategies.RecursiveChunk.html#produce_metadata/3","title":"TextChunker.Strategies.RecursiveChunk.produce_metadata/3","type":"function"},{"doc":"Splits the given text into chunks using a recursive strategy. Prioritizes compliance\nwith  the configured `chunk_size` as a maximum, while aiming to maintain `chunk_overlap` for\ncontext preservation.  Intelligently handles various separators for flexible splitting.","ref":"TextChunker.Strategies.RecursiveChunk.html#split/2","title":"TextChunker.Strategies.RecursiveChunk.split/2","type":"function"},{"doc":"* `:chunk_size` (integer) - Target size in bytes for each chunk.\n* `:chunk_overlap` (integer) - Number of overlapping bytes between chunks.\n* `:format` (atom) -  The format of the input text (influences separator selection).","ref":"TextChunker.Strategies.RecursiveChunk.html#split/2-options","title":"Options - TextChunker.Strategies.RecursiveChunk.split/2","type":"function"},{"doc":"```elixir\niex> long_text = \"This is a very long text that needs to be split into smaller pieces for easier handling.\"\n\niex> TextChunker.Strategies.RecursiveChunk.split(long_text, chunk_size: 15, chunk_overlap: 5)\n[\n  %TextChunker.Chunk{\n    start_byte: 0,\n    end_byte: 47,\n    text: \"This is a very long text that needs to be split\"\n  },\n  %TextChunker.Chunk{\n    start_byte: 38,\n    end_byte: 88,\n    text: \" be split into smaller pieces for easier handling.\"\n  }\n]\n```","ref":"TextChunker.Strategies.RecursiveChunk.html#split/2-examples","title":"Examples - TextChunker.Strategies.RecursiveChunk.split/2","type":"function"},{"doc":"Handles separator configuration for the RecursiveChunk text chunking strategy.\n\nProvides predefined lists of separators tailored for different text formats. The order of separators is crucial, as the splitting algorithm prioritizes them sequentially.\n\n**Key Features:**\n\n* **Format-Specific Separators:** Customizes splitting behavior based on formats like Markdown, plain text, Elixir, and others.\n* **Prioritized Splitting:** Attempts to split text using the highest-priority separator applicable to the text's content.","ref":"TextChunker.Strategies.RecursiveChunk.Separators.html","title":"TextChunker.Strategies.RecursiveChunk.Separators","type":"module"},{"doc":"Returns a list of separators that will be used to split the document of the given format","ref":"TextChunker.Strategies.RecursiveChunk.Separators.html#get_separators/1","title":"TextChunker.Strategies.RecursiveChunk.Separators.get_separators/1","type":"function"},{"doc":"# Contributor Covenant Code of Conduct","ref":"code_of_conduct.html","title":"Contributor Covenant Code of Conduct","type":"extras"},{"doc":"In the interest of fostering an open and welcoming environment, we as\ncontributors and maintainers pledge to making participation in our project and\nour community a harassment-free experience for everyone, regardless of age, body\nsize, disability, ethnicity, gender identity and expression, level of experience,\nnationality, personal appearance, race, religion, or sexual identity and\norientation.","ref":"code_of_conduct.html#our-pledge","title":"Our Pledge - Contributor Covenant Code of Conduct","type":"extras"},{"doc":"Examples of behavior that contributes to creating a positive environment\ninclude:\n\n* Using welcoming and inclusive language\n* Being respectful of differing viewpoints and experiences\n* Gracefully accepting constructive criticism\n* Focusing on what is best for the community\n* Showing empathy towards other community members\n\nExamples of unacceptable behavior by participants include:\n\n* The use of sexualized language or imagery and unwelcome sexual attention or\nadvances\n* Trolling, insulting/derogatory comments, and personal or political attacks\n* Public or private harassment\n* Publishing others' private information, such as a physical or electronic\n  address, without explicit permission\n* Other conduct which could reasonably be considered inappropriate in a\n  professional setting","ref":"code_of_conduct.html#our-standards","title":"Our Standards - Contributor Covenant Code of Conduct","type":"extras"},{"doc":"Project maintainers are responsible for clarifying the standards of acceptable\nbehavior and are expected to take appropriate and fair corrective action in\nresponse to any instances of unacceptable behavior.\n\nProject maintainers have the right and responsibility to remove, edit, or\nreject comments, commits, code, wiki edits, issues, and other contributions\nthat are not aligned to this Code of Conduct, or to ban temporarily or\npermanently any contributor for other behaviors that they deem inappropriate,\nthreatening, offensive, or harmful.","ref":"code_of_conduct.html#our-responsibilities","title":"Our Responsibilities - Contributor Covenant Code of Conduct","type":"extras"},{"doc":"This Code of Conduct applies both within project spaces and in public spaces\nwhen an individual is representing the project or its community. Examples of\nrepresenting a project or community include using an official project e-mail\naddress, posting via an official social media account, or acting as an appointed\nrepresentative at an online or offline event. Representation of a project may be\nfurther defined and clarified by project maintainers.","ref":"code_of_conduct.html#scope","title":"Scope - Contributor Covenant Code of Conduct","type":"extras"},{"doc":"Instances of abusive, harassing, or otherwise unacceptable behavior may be\nreported by contacting the project team at support@revelry.co. All\ncomplaints will be reviewed and investigated and will result in a response that\nis deemed necessary and appropriate to the circumstances. The project team is\nobligated to maintain confidentiality with regard to the reporter of an incident.\nFurther details of specific enforcement policies may be posted separately.\n\nProject maintainers who do not follow or enforce the Code of Conduct in good\nfaith may face temporary or permanent repercussions as determined by other\nmembers of the project's leadership.","ref":"code_of_conduct.html#enforcement","title":"Enforcement - Contributor Covenant Code of Conduct","type":"extras"},{"doc":"This Code of Conduct is adapted from the [Contributor Covenant][homepage], version 1.4,\navailable at [http://contributor-covenant.org/version/1/4][version]\n\n[homepage]: http://contributor-covenant.org\n[version]: http://contributor-covenant.org/version/1/4/","ref":"code_of_conduct.html#attribution","title":"Attribution - Contributor Covenant Code of Conduct","type":"extras"},{"doc":"# Releasing a new version","ref":"releases.html","title":"Releasing a new version","type":"extras"},{"doc":"- [ ] Update the package version in `mix.exs`\n- [ ] Update the package version badge and installation instructions in `README.md`","ref":"releases.html#preparation","title":"Preparation - Releasing a new version","type":"extras"},{"doc":"- [ ] Make sure all changes in the `Preparation` section are in `main`\n- [ ] In GitHub, create, and publish, a new release with appropriate notes, and version matching the package version `mix.exs`\n- [ ] The github actions workflow should automatically publish the package and docs to hex","ref":"releases.html#release-process","title":"Release Process - Releasing a new version","type":"extras"},{"doc":"# Contributing and Development","ref":"contributing.html","title":"Contributing and Development","type":"extras"},{"doc":"1. Make sure you have Elixir 1.15+ installed\n1. Clone the repo\n1. Run `mix deps.get`\n1. Run `mix test`","ref":"contributing.html#development-setup","title":"Development Setup - Contributing and Development","type":"extras"},{"doc":"1. Fork the project\n1. Create a new topic branch to contain your feature, change, or fix.\n1. Make sure all the tests are still passing.\n1. Implement your feature, change, or fix. Make sure to write tests, update and/or add documentation.\n1. Push your topic branch up to your fork.\n1. Open a Pull Request with a clear title and description.","ref":"contributing.html#submitting-changes","title":"Submitting Changes - Contributing and Development","type":"extras"},{"doc":"# TextChunker: Flexible Text Chunking for Elixir","ref":"readme.html","title":"TextChunker: Flexible Text Chunking for Elixir","type":"extras"},{"doc":"TextChunker is an Elixir library for segmenting large text documents, optimizing them for efficient embedding and storage within vector databases for use in retrieval augmented generation (RAG) applications.\n\nIt prioritizes context preservation and adaptability, and is therefore ideal for analytical, NLP, and other applications where understanding the relationship between text segments is crucial.","ref":"readme.html#about","title":"About - TextChunker: Flexible Text Chunking for Elixir","type":"extras"},{"doc":"Fill the gap in the Elixir ecosystem for a good semantic text chunker, and give access to langchain-style chunking.","ref":"readme.html#motivation","title":"Motivation - TextChunker: Flexible Text Chunking for Elixir","type":"extras"},{"doc":"![tests](https://github.com/revelrylabs/text_chunker_ex/actions/workflows/test.yml/badge.svg)","ref":"readme.html#build-status","title":"Build Status - TextChunker: Flexible Text Chunking for Elixir","type":"extras"},{"doc":"- Semantic Chunking: Prioritizes chunking text into meaningful blocks based on separators relevant to the specified format (e.g., headings, paragraphs in Markdown).\n- Configurable Chunking: Fine-tune the chunking process with options for, text chunk size, overlap and format.\n- Metadata Tracking: Automatically generates Chunk structs containing byte range information for accurately reassembling the original text if needed.\n- Extensibility: Designed to accommodate additional chunking strategies in the future.","ref":"readme.html#key-features","title":"Key Features - TextChunker: Flexible Text Chunking for Elixir","type":"extras"},{"doc":"Add TextChunker to your mix.exs:\n\n```elixir\ndef deps do\n  [\n    {:text_chunker, \"~> 0.3.0\"}\n  ]\nend\n```\n\nFetch dependencies:\n\n```\nmix deps.get\n```","ref":"readme.html#installation","title":"Installation - TextChunker: Flexible Text Chunking for Elixir","type":"extras"},{"doc":"Chunk your text using the `split` function:\n\n```elixir\ntext = \"Your text to be split...\"\n\nchunks = TextChunker.split(text)\n```\n\nThis will chunk up your text using the default parameters - a chunk size of `1000`, chunk overlap of `200`, format of `:plaintext` and using the `RecursiveChunk` strategy.\n\nThe split method returns `Chunks` of your text. These chunks include the start and end bytes of each chunk.\n\n```elixir\n%TextChunker.Chunk{\n    start_byte: 0,\n    end_byte: 44,\n    text: \"This is a sample text. It will be split into\",\n  }\n```","ref":"readme.html#usage","title":"Usage - TextChunker: Flexible Text Chunking for Elixir","type":"extras"},{"doc":"If you wish to adjust these parameters, configuration can optionally be passed via a keyword list. \n\n  - `chunk_size` -  The approximate target chunk size, as measured per code points. This means that both `a` and `👻` count as one. Chunks will not exceed this maximum, but may sometimes be smaller. **Important note** This means that graphemes *may* be split. For example, `👩‍🚒` may be split into `👩,🚒` or not depending on the split boundary.\n  - `chunk_overlap` - The contextual overlap between chunks, as measured per code point. Overlap is *not* guaranteed; again this should be treated as a maximum. The size of an individual overlap will depend on the semantics of the text being split.\n  - `format` - What informs separator selection. Because we are trying to preserve meaning between the chunks, the format of the text we are splitting is important. It's important to split newlines in plain text; it's important to split `###` headings in markdown.\n\n```elixir\ntext = \"\"\"","ref":"readme.html#options","title":"Options - TextChunker: Flexible Text Chunking for Elixir","type":"extras"},{"doc":"Let's split your text up properly!\n\"\"\"\nopts = [chunk_size: 10, chunk_overlap: 5, format: :markdown]\nchunks = TextChunker.split(text, opts)\n```","ref":"readme.html#your-text-to-be-split","title":"Your text to be split - TextChunker: Flexible Text Chunking for Elixir","type":"extras"},{"doc":"Currently, we only implement one strategy choice: Recursive Chunk. This was reverse-engineered from LangChain, with plans to add more methods in the future. \n\n#### Recursive Chunk (current default)\n\nYou can use Recursive Chunk to split text up into any chunk size you wish, with or without overlap. It is important to note that this overlap is not guaranteed - rather, if the overlap makes sense, this is the max length for that overlap. Recursive Chunk prioritizes keeping the semantics intact (as defined by the separators derived from the input format). The overlap does not occur when such an overlap would break those semantics. See below for examples.","ref":"readme.html#chunking-strategies","title":"Chunking Strategies - TextChunker: Flexible Text Chunking for Elixir","type":"extras"},{"doc":"```elixir\ntext = \"This is a sample text. It will be split into properly-sized chunks using the TextChunker library.\"\n\niex(10)> TextChunker.split(text)\n\n[\n  %TextChunker.Chunk{\n    start_byte: 0,\n    end_byte: 97,\n    text: \"This is a sample text. It will be split into properly-sized chunks using the TextChunker library.\"\n  }\n]\n\ntext = \"This is a sample text. It will be split into properly-sized chunks using the TextChunker library.\"\nopts = [chunk_size: 50, chunk_overlap: 5, format: :plaintext, strategy: TextChunker.Strategies.RecursiveChunk]\n\niex(10)> TextChunker.split(text, opts)\n\n[\n  %TextChunker.Chunk{\n    start_byte: 0,\n    end_byte: 44,\n    text: \"This is a sample text. It will be split into\"\n  },\n  %TextChunker.Chunk{\n    start_byte: 39,\n    end_byte: 88,\n    text: \" into properly-sized chunks using the TextChunker\"\n  },\n  %TextChunker.Chunk{\n    start_byte: 88,\n    end_byte: 97,\n    text: \" library.\"\n  }\n]\n```","ref":"readme.html#examples","title":"Examples - TextChunker: Flexible Text Chunking for Elixir","type":"extras"},{"doc":"Bug reports and pull requests are welcome on GitHub at https://github.com/revelrylabs/text_chunker_ex. Check out the [contributing guidelines](CONTRIBUTING.md) for more info.\n\nEveryone is welcome to participate in the project. We expect contributors to adhere to the [Contributor Covenant Code of Conduct](CODE_OF_CONDUCT.md).","ref":"readme.html#contributing-and-development","title":"Contributing and Development - TextChunker: Flexible Text Chunking for Elixir","type":"extras"},{"doc":"See [RELEASES.md](RELEASES.md) for details about the release process.","ref":"readme.html#releases","title":"Releases - TextChunker: Flexible Text Chunking for Elixir","type":"extras"},{"doc":"Special thanks to the creators of Langchain for their initial approach to recursive text splitting, which inspired this library. See the [NOTICE](NOTICE) file for details.","ref":"readme.html#acknowledgments","title":"Acknowledgments - TextChunker: Flexible Text Chunking for Elixir","type":"extras"},{"doc":"TextChunker is released under the MIT License. See the [LICENSE](LICENSE) file for details.","ref":"readme.html#license","title":"License - TextChunker: Flexible Text Chunking for Elixir","type":"extras"},{"doc":"MIT License\n\nCopyright (c) 2024 Revelry\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html","title":"LICENSE","type":"extras"},{"doc":"Portions of this project are derived from work by Harrison Chase, specifically from the langchainjs library, under the MIT License.\n\nThe MIT License\n\nCopyright (c) Harrison Chase\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in\nall copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN\nTHE SOFTWARE.","ref":"notice.html","title":"NOTICE","type":"extras"}],"producer":{"name":"ex_doc","version":[48,46,51,50,46,48]}}